@@ -42,8 +42,8 @@ symbol (example: `python bootstrap.py -- --hide-console`).
 Type `python bootstrap.py -- --help` to read about Spyder
 options.""")
 parser.add_argument('--gui', default=None,
-                    help="GUI toolkit: pyqt5 (for PyQt5), pyqt (for PyQt4) or "
-                    "pyside (for PySide, deprecated)")
+                    help="GUI toolkit: pyqt5 (for PyQt5) or pyside2 "
+                    "(for PySide2)")
 parser.add_argument('--show-console', action='store_true', default=False,
                     help="(Deprecated) Does nothing, now the default behavior "
                     "is to show the console")
@@ -67,7 +67,7 @@ args = parser.parse_args()
 # Store variable to be used in self.restart (restart spyder instance)
 os.environ['SPYDER_BOOTSTRAP_ARGS'] = str(sys.argv[1:])
 
-assert args.gui in (None, 'pyqt5', 'pyqt', 'pyside'), \
+assert args.gui in (None, 'pyqt5', 'pyside2'), \
        "Invalid GUI toolkit option '%s'" % args.gui
 
 # Start Spyder with a clean configuration directory for testing purposes
