@@ -19,27 +19,34 @@ import time
 from subprocess import Popen, PIPE
 import sys
 import inspect
+from collections import namedtuple
 
 # Test imports
+import ipykernel
 import IPython
 import pytest
 from flaky import flaky
 from jupyter_core import paths
 from jupyter_client import BlockingKernelClient
-from ipython_genutils import py3compat
 import numpy as np
 
-
 # Local imports
-from spyder_kernels.py3compat import PY3, to_text_string
+from spyder_kernels.py3compat import PY2, PY3, to_text_string
 from spyder_kernels.utils.iofuncs import iofunctions
 from spyder_kernels.utils.test_utils import get_kernel, get_log_text
 from spyder_kernels.customize.spyderpdb import SpyderPdb
 
+# For ipykernel 6
+try:
+    import asyncio
+except ImportError:
+    pass
+
 # =============================================================================
 # Constants
 # =============================================================================
 FILES_PATH = os.path.dirname(os.path.realpath(__file__))
+IPYKERNEL_6 = ipykernel.__version__[0] >= '6'
 TIMEOUT = 15
 SETUP_TIMEOUT = 60
 
@@ -51,9 +58,9 @@ def setup_kernel(cmd):
     This function was taken from the ipykernel project.
     We plan to remove it when dropping support for python 2.
 
-    Returns
+    Yields
     -------
-    kernel_manager: connected KernelManager instance
+    client: jupyter_client.BlockingKernelClient connected to the kernel
     """
     kernel = Popen([sys.executable, '-c', cmd], stdout=PIPE, stderr=PIPE)
     try:
@@ -70,7 +77,8 @@ def setup_kernel(cmd):
 
         if kernel.poll() is not None:
             o,e = kernel.communicate()
-            e = py3compat.cast_unicode(e)
+            if not PY3 and isinstance(e, bytes):
+                e = e.decode()
             raise IOError("Kernel failed to start:\n%s" % e)
 
         if not os.path.exists(connection_file):
@@ -131,7 +139,10 @@ def kernel(request):
 
     # Teardown
     def reset_kernel():
-        kernel.do_execute('reset -f', True)
+        if IPYKERNEL_6:
+            asyncio.run(kernel.do_execute('reset -f', True))
+        else:
+            kernel.do_execute('reset -f', True)
     request.addfinalizer(reset_kernel)
 
     return kernel
@@ -176,7 +187,10 @@ def test_get_namespace_view(kernel):
     """
     Test the namespace view of the kernel.
     """
-    execute = kernel.do_execute('a = 1', True)
+    if IPYKERNEL_6:
+        execute = asyncio.run(kernel.do_execute('a = 1', True))
+    else:
+        execute = kernel.do_execute('a = 1', True)
 
     nsview = repr(kernel.get_namespace_view())
     assert "'a':" in nsview
@@ -195,7 +209,10 @@ def test_get_var_properties(kernel):
     """
     Test the properties fo the variables in the namespace.
     """
-    execute = kernel.do_execute('a = 1', True)
+    if IPYKERNEL_6:
+        asyncio.run(kernel.do_execute('a = 1', True))
+    else:
+        kernel.do_execute('a = 1', True)
 
     var_properties = repr(kernel.get_var_properties())
     assert "'a'" in var_properties
@@ -213,7 +230,10 @@ def test_get_var_properties(kernel):
 def test_get_value(kernel):
     """Test getting the value of a variable."""
     name = 'a'
-    kernel.do_execute("a = 124", True)
+    if IPYKERNEL_6:
+        asyncio.run(kernel.do_execute("a = 124", True))
+    else:
+        kernel.do_execute("a = 124", True)
 
     # Check data type send
     assert kernel.get_value(name) == 124
@@ -222,7 +242,10 @@ def test_get_value(kernel):
 def test_set_value(kernel):
     """Test setting the value of a variable."""
     name = 'a'
-    execute = kernel.do_execute('a = 0', True)
+    if IPYKERNEL_6:
+         asyncio.run(kernel.do_execute('a = 0', True))
+    else:
+        kernel.do_execute('a = 0', True)
     value = 10
     kernel.set_value(name, value)
     log_text = get_log_text(kernel)
@@ -236,7 +259,10 @@ def test_set_value(kernel):
 def test_remove_value(kernel):
     """Test the removal of a variable."""
     name = 'a'
-    execute = kernel.do_execute('a = 1', True)
+    if IPYKERNEL_6:
+        asyncio.run(kernel.do_execute('a = 1', True))
+    else:
+        kernel.do_execute('a = 1', True)
 
     var_properties = repr(kernel.get_var_properties())
     assert "'a'" in var_properties
@@ -258,7 +284,10 @@ def test_copy_value(kernel):
     """Test the copy of a variable."""
     orig_name = 'a'
     new_name = 'b'
-    execute = kernel.do_execute('a = 1', True)
+    if IPYKERNEL_6:
+         asyncio.run(kernel.do_execute('a = 1', True))
+    else:
+        kernel.do_execute('a = 1', True)
 
     var_properties = repr(kernel.get_var_properties())
     assert "'a'" in var_properties
@@ -294,7 +323,12 @@ def test_load_npz_data(kernel, load):
     namespace_file = osp.join(FILES_PATH, 'load_data.npz')
     extention = '.npz'
     overwrite, execute, variables = load
-    kernel.do_execute(execute, True)
+
+    if IPYKERNEL_6:
+        asyncio.run(kernel.do_execute(execute, True))
+    else:
+        kernel.do_execute(execute, True)
+
     kernel.load_data(namespace_file, extention, overwrite=overwrite)
     for var, value in variables.items():
         assert value == kernel.get_value(var)
@@ -321,7 +355,11 @@ def test_load_data(kernel):
 def test_save_namespace(kernel):
     """Test saving the namespace into filename."""
     namespace_file = osp.join(FILES_PATH, 'save_data.spydata')
-    execute = kernel.do_execute('b = 1', True)
+
+    if IPYKERNEL_6:
+        asyncio.run(kernel.do_execute('b = 1', True))
+    else:
+        kernel.do_execute('b = 1', True)
 
     kernel.save_namespace(namespace_file)
     assert osp.isfile(namespace_file)
@@ -365,7 +403,11 @@ libc.printf(('Hello from C\\n').encode('utf8'))
 
     # With Wurlitzer we have the expected output
     kernel._load_wurlitzer()
-    reply = kernel.do_execute(code, True)
+
+    if IPYKERNEL_6:
+        asyncio.run(kernel.do_execute(code, True))
+    else:
+        kernel.do_execute(code, True)
     captured = capsys.readouterr()
     assert captured.out == "Hello from C\n"
 
@@ -382,7 +424,9 @@ def test_cwd_in_sys_path():
     with setup_kernel(cmd) as client:
         msg_id = client.execute("import sys; sys_path = sys.path",
                                 user_expressions={'output':'sys_path'})
-        reply = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        reply = client.get_shell_msg(timeout=TIMEOUT)
+        while 'user_expressions' not in reply['content']:
+            reply = client.get_shell_msg(timeout=TIMEOUT)
 
         # Transform value obtained through user_expressions
         user_expressions = reply['content']['user_expressions']
@@ -406,7 +450,7 @@ def test_multiprocessing(tmpdir):
     with setup_kernel(cmd) as client:
         # Remove all variables
         client.execute("%reset -f")
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Write multiprocessing code to a file
         code = """
@@ -424,15 +468,65 @@ if __name__ == '__main__':
 
         # Run code
         client.execute("runfile(r'{}')".format(to_text_string(p)))
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Verify that the `result` variable is defined
         client.inspect('result')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
         assert content['found']
 
 
+@flaky(max_runs=3)
+@pytest.mark.skipif(not PY3,
+                    reason="Only meant for Python 3")
+def test_multiprocessing_2(tmpdir):
+    """
+    Test that multiprocessing works on Python 3.
+    """
+    # Command to start the kernel
+    cmd = "from spyder_kernels.console import start; start.main()"
+
+    with setup_kernel(cmd) as client:
+        # Remove all variables
+        client.execute("%reset -f")
+        client.get_shell_msg(timeout=TIMEOUT)
+
+        # Write multiprocessing code to a file
+        code = """
+from multiprocessing import Pool
+
+class myClass():
+    def __init__(self, i):
+        self.i = i + 10
+
+def myFunc(i):
+    return myClass(i)
+
+if __name__ == '__main__':
+    with Pool(5) as p:
+        result = p.map(myFunc, [1, 2, 3])
+    result = [r.i for r in result]
+"""
+        p = tmpdir.join("mp-test.py")
+        p.write(code)
+
+        # Run code
+        client.execute("runfile(r'{}')".format(to_text_string(p)))
+        client.get_shell_msg(timeout=TIMEOUT)
+
+        # Verify that the `result` variable is defined
+        client.inspect('result')
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
+        content = msg['content']
+        assert content['found']
+        assert "[11, 12, 13]" in content['data']['text/plain']
+
+
 @flaky(max_runs=3)
 @pytest.mark.skipif(not PY3,
                     reason="Only meant for Python 3")
@@ -446,7 +540,7 @@ def test_dask_multiprocessing(tmpdir):
     with setup_kernel(cmd) as client:
         # Remove all variables
         client.execute("%reset -f")
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Write multiprocessing code to a file
         # Runs two times to verify that in the second case it doesn't break
@@ -463,14 +557,16 @@ if __name__=='__main__':
 
         # Run code two times
         client.execute("runfile(r'{}')".format(to_text_string(p)))
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         client.execute("runfile(r'{}')".format(to_text_string(p)))
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Verify that the `x` variable is defined
         client.inspect('x')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
         assert content['found']
 
@@ -486,7 +582,7 @@ def test_runfile(tmpdir):
     with setup_kernel(cmd) as client:
         # Remove all variables
         client.execute("%reset -f")
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Write defined variable code to a file
         code = u"result = 'hello world'; error # make an error"
@@ -506,40 +602,48 @@ def test_runfile(tmpdir):
         # Run code file `d` to define `result` even after an error
         client.execute("runfile(r'{}', current_namespace=False)"
                        .format(to_text_string(d)))
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Verify that `result` is defined in the current namespace
         client.inspect('result')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
         assert content['found']
 
         # Run code file `u` without current namespace
         client.execute("runfile(r'{}', current_namespace=False)"
                        .format(to_text_string(u)))
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Verify that the variable `result2` is defined
         client.inspect('result2')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
         assert content['found']
 
         # Run code file `u` with current namespace
         client.execute("runfile(r'{}', current_namespace=True)"
                        .format(to_text_string(u)))
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
 
         # Verify that the variable `result3` is defined
         client.inspect('result3')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
         assert content['found']
 
         # Verify that the variable `__file__` is undefined
         client.inspect('__file__')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
         assert not content['found']
 
@@ -560,14 +664,14 @@ np.set_printoptions(
     suppress=True,
     formatter={'float_kind':'{:0.2f}'.format})
     """)
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Create a big Numpy array and an array to check decimal format
         client.execute("""
 x = np.random.rand(75000,5);
 a = np.array([123412341234.123412341234])
 """)
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Assert that NumPy threshold, suppress and formatter
         # are the same as the ones set by the user
@@ -576,29 +680,37 @@ t = np.get_printoptions()['threshold'];
 s = np.get_printoptions()['suppress'];
 f = np.get_printoptions()['formatter']
 """)
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Check correct decimal format
         client.inspect('a')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "data" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']['data']['text/plain']
         assert "123412341234.12" in content
 
         # Check threshold value
         client.inspect('t')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "data" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']['data']['text/plain']
         assert "inf" in content
 
         # Check suppress value
         client.inspect('s')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "data" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']['data']['text/plain']
         assert "True" in content
 
         # Check formatter
         client.inspect('f')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "data" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']['data']['text/plain']
         assert "{'float_kind': <built-in method format of str object" in content
 
@@ -626,7 +738,7 @@ def test_turtle_launch(tmpdir):
     with setup_kernel(cmd) as client:
         # Remove all variables
         client.execute("%reset -f")
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Write turtle code to a file
         code = """
@@ -651,11 +763,13 @@ turtle.bye()
 
         # Run code
         client.execute("runfile(r'{}')".format(to_text_string(p)))
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Verify that the `tess` variable is defined
         client.inspect('tess')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
         assert content['found']
 
@@ -667,11 +781,13 @@ turtle.bye()
 
         # Run code again
         client.execute("runfile(r'{}')".format(to_text_string(p)))
-        client.get_shell_msg(block=True, timeout=TIMEOUT)
+        client.get_shell_msg(timeout=TIMEOUT)
 
         # Verify that the `a` variable is defined
         client.inspect('a')
-        msg = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        msg = client.get_shell_msg(timeout=TIMEOUT)
+        while "found" not in msg['content']:
+            msg = client.get_shell_msg(timeout=TIMEOUT)
         content = msg['content']
         assert content['found']
 
@@ -686,7 +802,9 @@ def test_matplotlib_inline(kernel):
         # Get current backend
         code = "import matplotlib; backend = matplotlib.get_backend()"
         client.execute(code, user_expressions={'output': 'backend'})
-        reply = client.get_shell_msg(block=True, timeout=TIMEOUT)
+        reply = client.get_shell_msg(timeout=TIMEOUT)
+        while 'user_expressions' not in reply['content']:
+            reply = client.get_shell_msg(timeout=TIMEOUT)
 
         # Transform value obtained through user_expressions
         user_expressions = reply['content']['user_expressions']
@@ -701,7 +819,10 @@ def test_do_complete(kernel):
     """
     Check do complete works in normal and debugging mode.
     """
-    kernel.do_execute('abba = 1', True)
+    if IPYKERNEL_6:
+        asyncio.run(kernel.do_execute('abba = 1', True))
+    else:
+        kernel.do_execute('abba = 1', True)
     assert kernel.get_value('abba') == 1
     match = kernel.do_complete('ab', 2)
     assert 'abba' in match['matches']
@@ -710,9 +831,10 @@ def test_do_complete(kernel):
     pdb_obj = SpyderPdb()
     pdb_obj.curframe = inspect.currentframe()
     pdb_obj.completenames = lambda *ignore: ['baba']
-    kernel._pdb_obj = pdb_obj
+    kernel.shell.pdb_session = pdb_obj
     match = kernel.do_complete('ba', 2)
     assert 'baba' in match['matches']
+    pdb_obj.curframe = None
 
 
 @pytest.mark.parametrize("exclude_callables_and_modules", [True, False])
@@ -723,9 +845,15 @@ def test_callables_and_modules(kernel, exclude_callables_and_modules,
     Tests that callables and modules are in the namespace view only
     when the right options are passed to the kernel.
     """
-    kernel.do_execute('import numpy', True)
-    kernel.do_execute('a = 10', True)
-    kernel.do_execute('def f(x): return x', True)
+    if IPYKERNEL_6:
+        asyncio.run(kernel.do_execute('import numpy', True))
+        asyncio.run(kernel.do_execute('a = 10', True))
+        asyncio.run(kernel.do_execute('def f(x): return x', True))
+    else:
+        kernel.do_execute('import numpy', True)
+        kernel.do_execute('a = 10', True)
+        kernel.do_execute('def f(x): return x', True)
+
     settings = kernel.namespace_view_settings
 
     settings['exclude_callables_and_modules'] = exclude_callables_and_modules
@@ -761,20 +889,209 @@ def test_comprehensions_with_locals_in_pdb(kernel):
     pdb_obj = SpyderPdb()
     pdb_obj.curframe = inspect.currentframe()
     pdb_obj.curframe_locals = pdb_obj.curframe.f_locals
-    kernel._pdb_obj = pdb_obj
+    kernel.shell.pdb_session = pdb_obj
 
     # Create a local variable.
-    kernel._pdb_obj.default('zz = 10')
+    kernel.shell.pdb_session.default('zz = 10')
     assert kernel.get_value('zz') == 10
 
     # Run a list comprehension with this variable.
-    kernel._pdb_obj.default("compr = [zz * i for i in [1, 2, 3]]")
+    kernel.shell.pdb_session.default("compr = [zz * i for i in [1, 2, 3]]")
     assert kernel.get_value('compr') == [10, 20, 30]
 
     # Check that the variable is not reported as being part of globals.
-    kernel._pdb_obj.default("in_globals = 'zz' in globals()")
+    kernel.shell.pdb_session.default("in_globals = 'zz' in globals()")
     assert kernel.get_value('in_globals') == False
 
+    pdb_obj.curframe = None
+    pdb_obj.curframe_locals = None
+
+def test_comprehensions_with_locals_in_pdb_2(kernel):
+    """
+    Test that evaluating comprehensions with locals works in Pdb.
+
+    This is a regression test for spyder-ide/spyder#16790.
+    """
+    pdb_obj = SpyderPdb()
+    pdb_obj.curframe = inspect.currentframe()
+    pdb_obj.curframe_locals = pdb_obj.curframe.f_locals
+    kernel.shell.pdb_session = pdb_obj
+
+    # Create a local variable.
+    kernel.shell.pdb_session.default('aa = [1, 2]')
+    kernel.shell.pdb_session.default('bb = [3, 4]')
+    kernel.shell.pdb_session.default('res = []')
+
+    # Run a list comprehension with this variable.
+    kernel.shell.pdb_session.default(
+        "for c0 in aa: res.append([(c0, c1) for c1 in bb])")
+    assert kernel.get_value('res') == [[(1, 3), (1, 4)], [(2, 3), (2, 4)]]
+
+    pdb_obj.curframe = None
+    pdb_obj.curframe_locals = None
+
+
+def test_namespaces_in_pdb(kernel):
+    """
+    Test namespaces in pdb
+    """
+    # Define get_ipython for timeit
+    get_ipython = lambda: kernel.shell
+    kernel.shell.user_ns["test"] = 0
+    pdb_obj = SpyderPdb()
+    pdb_obj.curframe = inspect.currentframe()
+    pdb_obj.curframe_locals = pdb_obj.curframe.f_locals
+    kernel.shell.pdb_session = pdb_obj
+
+    # Check adding something to globals works
+    pdb_obj.default("globals()['test2'] = 0")
+    assert pdb_obj.curframe.f_globals["test2"] == 0
+
+    if PY2:
+        # no error method in py2
+        pdb_obj.curframe = None
+        pdb_obj.curframe_locals = None
+        return
+
+    # Create wrapper to check for errors
+    old_error = pdb_obj.error
+    pdb_obj._error_occured = False
+    def error_wrapper(*args, **kwargs):
+        print(args, kwargs)
+        pdb_obj._error_occured = True
+        return old_error(*args, **kwargs)
+    pdb_obj.error = error_wrapper
+
+    # Test globals are visible
+    pdb_obj.curframe.f_globals["test3"] = 0
+    pdb_obj.default("%timeit test3")
+    assert not pdb_obj._error_occured
+
+    # Test locals are visible
+    pdb_obj.curframe_locals["test4"] = 0
+    pdb_obj.default("%timeit test4")
+    assert not pdb_obj._error_occured
+
+    # Test user namespace is not visible
+    pdb_obj.default("%timeit test")
+    assert pdb_obj._error_occured
+
+    pdb_obj.curframe = None
+    pdb_obj.curframe_locals = None
+
+
+def test_functions_with_locals_in_pdb(kernel):
+    """
+    Test that functions with locals work in Pdb.
+
+    This is a regression test for spyder-ide/spyder-kernels#345
+    """
+    pdb_obj = SpyderPdb()
+    Frame = namedtuple("Frame", ["f_globals"])
+    pdb_obj.curframe = Frame(f_globals=kernel.shell.user_ns)
+    pdb_obj.curframe_locals = kernel.shell.user_ns
+    kernel.shell.pdb_session = pdb_obj
+
+    # Create a local function.
+    kernel.shell.pdb_session.default(
+        'def fun_a(): return [i for i in range(1)]')
+    kernel.shell.pdb_session.default(
+        'zz = fun_a()')
+    assert kernel.get_value('zz') == [0]
+
+    kernel.shell.pdb_session.default(
+        'a = 1')
+    kernel.shell.pdb_session.default(
+        'def fun_a(): return a')
+    kernel.shell.pdb_session.default(
+        'zz = fun_a()')
+    assert kernel.get_value('zz') == 1
+
+
+    pdb_obj.curframe = None
+    pdb_obj.curframe_locals = None
+
+
+def test_functions_with_locals_in_pdb_2(kernel):
+    """
+    Test that functions with locals work in Pdb.
+
+    This is another regression test for spyder-ide/spyder-kernels#345
+    """
+    baba = 1
+    pdb_obj = SpyderPdb()
+    pdb_obj.curframe = inspect.currentframe()
+    pdb_obj.curframe_locals = pdb_obj.curframe.f_locals
+    kernel.shell.pdb_session = pdb_obj
+
+    # Create a local function.
+    kernel.shell.pdb_session.default(
+        'def fun_a(): return [i for i in range(1)]')
+    kernel.shell.pdb_session.default(
+        'zz = fun_a()')
+    assert kernel.get_value('zz') == [0]
+
+    kernel.shell.pdb_session.default(
+        'a = 1')
+    kernel.shell.pdb_session.default(
+        'def fun_a(): return a')
+    kernel.shell.pdb_session.default(
+        'zz = fun_a()')
+    assert kernel.get_value('zz') == 1
+
+    # Check baba is in locals and not globals
+    kernel.shell.pdb_session.default(
+        'll = locals().keys()')
+    assert "baba" in kernel.get_value('ll')
+    kernel.shell.pdb_session.default(
+        'gg = globals().keys()')
+    assert "baba" not in kernel.get_value('gg')
+
+    pdb_obj.curframe = None
+    pdb_obj.curframe_locals = None
+
+
+def test_locals_globals_in_pdb(kernel):
+    """
+    Test thal locals and globals work properly in Pdb.
+    """
+    a = 1
+    pdb_obj = SpyderPdb()
+    pdb_obj.curframe = inspect.currentframe()
+    pdb_obj.curframe_locals = pdb_obj.curframe.f_locals
+    kernel.shell.pdb_session = pdb_obj
+
+    assert kernel.get_value('a') == 1
+
+    kernel.shell.pdb_session.default(
+        'test = "a" in globals()')
+    assert kernel.get_value('test') == False
+
+    kernel.shell.pdb_session.default(
+        'test = "a" in locals()')
+    assert kernel.get_value('test') == True
+
+    kernel.shell.pdb_session.default(
+        'def f(): return a')
+    kernel.shell.pdb_session.default(
+        'test = f()')
+    assert kernel.get_value('test') == 1
+
+    kernel.shell.pdb_session.default(
+        'a = 2')
+    assert kernel.get_value('a') == 2
+
+    kernel.shell.pdb_session.default(
+        'test = "a" in globals()')
+    assert kernel.get_value('test') == False
+
+    kernel.shell.pdb_session.default(
+        'test = "a" in locals()')
+    assert kernel.get_value('test') == True
+
+    pdb_obj.curframe = None
+    pdb_obj.curframe_locals = None
+
 
 if __name__ == "__main__":
     pytest.main()
